//===============================================================
//
// $Source: /pv/CvsTree/pv/gen/src/prg/shimserver/OrbWrapper.cc,v $
//
// Copyright (c) 2009
//
// Bruker BioSpin MRI GmbH
// D-76275 Ettlingen, Germany
//
// All Rights Reserved
//===============================================================

static const char resid[] = "$Id: OrbWrapper.cc,v 1.1 2009/02/27 16:23:59 mgoe Exp $ (C) 2009 Bruker BioSpin MRI GmbH";

#define DEBUG 1

//===============================================================
//
// Description :
//
//===============================================================

//==============================================================//
//==============================================================//
//		I N T E R F A C E   S E C T I O N		//
//==============================================================//
//==============================================================//

//==============================================================//
//		I N C L U D E   F I L E S			//
//==============================================================//

//--------------------------------------------------------------
// system include files...
//--------------------------------------------------------------
#include <string>
#include <iostream>
#include <sstream>

//--------------------------------------------------------------
// Uxnmr include files...
//--------------------------------------------------------------

//--------------------------------------------------------------
// local include files...
//--------------------------------------------------------------
#include "OrbWrapper.hh"

//==============================================================//
//		E X T E R N A L   F U N C T I O N S		//
//==============================================================//


//==============================================================//
//		E X T E R N A L   V A R I A B L E S		//
//==============================================================//


//==============================================================//
//		G L O B A L   V A R I A B L E S			//
//==============================================================//


//==============================================================//
//==============================================================//
//	I M P L E M E N T A T I O N   S E C T I O N		//
//==============================================================//
//==============================================================//

//==============================================================//
//		L O C A L   D E F I N I T I O N S		//
//==============================================================//

//==============================================================//
//	L O C A L   F U N C T I O N   P R O T O T Y P E S	//
//			forward references			//
//==============================================================//

//==============================================================//
//		C L A S S   V A R I A B L E S			//
//==============================================================//

//==============================================================//
//		C O N S T R U C T O R S				//
//==============================================================//
OrbWrapper::OrbWrapper(CORBA::ORB_ptr orb, PortableServer::POA_ptr rootPOA) {
#if DEBUG
    std::cout << "OrbWrapper::OrbWrapper() ..." << std::endl;
#endif

    this->orb = orb;
    this->rootPOA = rootPOA;

#if DEBUG
    std::cout << "OrbWrapper::OrbWrapper() done" << std::endl;
#endif
}

//==============================================================//
//		D E S T R U C T O R				//
//==============================================================//
OrbWrapper::~OrbWrapper() {
#if DEBUG
    std::cout << "OrbWrapper::~OrbWrapper() ..." << std::endl;
#endif

    cleanup(this->orb, this->rootPOA);

#if DEBUG
    std::cout << "OrbWrapper::~OrbWrapper() done" << std::endl;
#endif
}

//==============================================================//
//		M E T H O D S					//
//==============================================================//

OrbWrapper* OrbWrapper::createInstance(
        const char *nameserver,
        const char *endpoint,
        unsigned int outConScanPeriod,
        unsigned int inConScanPeriod,
        unsigned int scanGranularity,
        unsigned int clientCallTimeOutPeriod,
        unsigned int clientConnectTimeOutPeriod) {
#if DEBUG
    std::cout << "OrbWrapper::createInstance("
        << "nameserver=" << nameserver
        << ", endpoint=" << endpoint
        << ", outConScanPeriod=" << outConScanPeriod
        << ", inConScanPeriod=" << inConScanPeriod
        << ", scanGranularity=" << scanGranularity
        << ", clientCallTimeOutPeriod=" << clientCallTimeOutPeriod
        << ", clientConnectTimeOutPeriod=" << clientConnectTimeOutPeriod
        << ") ..." << std::endl;
#endif

    int argc =21;
    char* argv[21];
    std::stringstream ss;
    ss<<"NameService="<<nameserver;
    const std::string sss(ss.str());
    argv[0] = const_cast<char*>("dummy_app_name");
    argv[1] = const_cast<char*>("-ORBInitRef");
    argv[2] = const_cast<char*>(sss.c_str());
    argv[3] = const_cast<char*>("-ORBtraceLevel");
    argv[4] = const_cast<char*>("40");
    argv[5] = const_cast<char*>("-ORBtraceThreadId");
    argv[6] = const_cast<char*>("1");

    // The idle timeout in seconds for outgoing (i.e. client initiated)
    // connections. If a connection has been idle for this amount of time,
    // the ORB closes it.
    argv[7] = const_cast<char*>("-ORBoutConScanPeriod");
    std::ostringstream outConScanPeriodSS;
    outConScanPeriodSS << outConScanPeriod;
    const std::string outConScanPeriodString(outConScanPeriodSS.str());
    argv[8] = const_cast<char*>(outConScanPeriodString.c_str());

    // The idle timeout in seconds for incoming connections.
    // If a connection has been idle for this amount of time, the ORB closes it.
    argv[9] = const_cast<char*>("-ORBinConScanPeriod");
    std::ostringstream inConScanPeriodSS;
    inConScanPeriodSS << inConScanPeriod;
    const std::string inConScanPeriodString(inConScanPeriodSS.str());
    argv[10] = const_cast<char*>(inConScanPeriodString.c_str());

    // omniORB regularly scans incoming and outgoing connections, so it can
    // close unused ones. This value is the granularity in seconds at which the
    // ORB performs its scans. A value of zero turns off the scanning
    // altogether.
    argv[11] = const_cast<char*>("-ORBscanGranularity");
    std::ostringstream scanGranularitySS;
    scanGranularitySS << scanGranularity;
    const std::string scanGranularityString(scanGranularitySS.str());
    argv[12] = const_cast<char*>(scanGranularityString.c_str());

    // The largest message, in bytes, that the ORB will send or receive,
    // to avoid resource starvation. If the limit is exceeded, a MARSHAL
    // exception is thrown. The size must be >= 8192.
    // CAUTION: There is a java property too (Orb.java):
    // yoko.orb.giop.max_message_size
    argv[13] = const_cast<char*>("-ORBgiopMaxMsgSize");
    argv[14] = const_cast<char*>("16777216");

    if (endpoint) {
        argv[15] = const_cast<char*>("-ORBendPoint");
        argv[16] = const_cast<char*>(endpoint);
    } else {
        // This is necessary to setup ipv4 only. Without  RHEL 3 will not work.
        argv[15] = const_cast<char*>("-ORBendPoint");
        argv[16] = const_cast<char*>("giop:tcp:0.0.0.0:");
    }

    // The timeout in milliseconds for the client side. If a call takes longer
    // than the specified number of milliseconds, the ORB closes the connection
    // to the server and raises a TRANSIENT exception. A value of zero means
    // no timeout; calls can block for ever.
    argv[17] = const_cast<char*>("-ORBclientCallTimeOutPeriod");
    std::ostringstream clientCallTimeOutPeriodSS;
    clientCallTimeOutPeriodSS << clientCallTimeOutPeriod;
    const std::string clientCallTimeOutPeriodString(
        clientCallTimeOutPeriodSS.str());
    argv[18] = const_cast<char*>(clientCallTimeOutPeriodString.c_str());

    // The timeout in milliseconds that is used in the case that a new network
    // connection is established to the server. A value of zero means that the
    // normal call timeout is used.
    argv[19] = const_cast<char*>("-ORBclientConnectTimeOutPeriod");
    std::ostringstream clientConnectTimeOutPeriodSS;
    clientConnectTimeOutPeriodSS << clientConnectTimeOutPeriod;
    const std::string clientConnectTimeOutPeriodString(
        clientConnectTimeOutPeriodSS.str());
    argv[20] = const_cast<char*>(clientConnectTimeOutPeriodString.c_str());

    OrbWrapper* orbWrapper = 0;
    try {
        // Try to create ORB
        CORBA::ORB_var orb = CORBA::ORB_init(argc, argv, "omniORB4");

        try {
            // Try to get the poa object for the root poa
            CORBA::Object_var poaObj =
                orb->resolve_initial_references("RootPOA");

            try {
                // Try to cast poa object to poa
                PortableServer::POA_var poa =
                    PortableServer::POA::_narrow(poaObj);

                try {
                    // Try to activate the poa
                    PortableServer::POAManager_var poaManager =
                        poa->the_POAManager();
                    poaManager->activate();

                    // We have an orb with an activated root poa
                    orbWrapper = new OrbWrapper(orb._retn(), poa._retn());
                } catch (const CORBA::Exception& ex) {
                    std::cerr << "CORBA::Exception caught while "
                        << "trying to activate root poa" << std::endl;
                    cleanup(orb, poa);
                } catch (...) {
                    std::cerr << "Exception caught while "
                        << "trying to activate root poa" << std::endl;
                    cleanup(orb, poa);
                }

            } catch (const CORBA::Exception& ex) {
                std::cerr << "CORBA::Exception caught while calling "
                    << "PortableServer::POA::_narrow(poaObj)" << std::endl;
                cleanup(orb, PortableServer::POA::_nil());
            } catch (...) {
                std::cerr << "Exception caught while calling "
                    << "PortableServer::POA::_narrow(poaObj)" << std::endl;
                cleanup(orb, PortableServer::POA::_nil());
            }

        } catch (const CORBA::Exception& ex) {
            std::cerr << "CORBA::Exception caught while calling "
                << "orb->resolve_initial_references()" << std::endl;
            cleanup(orb, PortableServer::POA::_nil());
        } catch (...) {
            std::cerr << "Exception caught while calling "
                << "orb->resolve_initial_references()" << std::endl;
            cleanup(orb, PortableServer::POA::_nil());
        }

    } catch (const CORBA::Exception& ex) {
        std::cerr << "CORBA::Exception caught while calling "
            << "CORBA::ORB_init()" << std::endl;
    } catch (...) {
        std::cerr << "Exception caught while calling "
            << "CORBA::ORB_init()" << std::endl;
    }

#if DEBUG
    std::cout << "OrbWrapper::createInstance() = " << orbWrapper << " done"
        << std::endl;
#endif
    return orbWrapper;
}

CORBA::ORB_var OrbWrapper::getORB() {
#if DEBUG
    std::cout << "OrbWrapper::getORB() ..." << std::endl;
#endif

#if DEBUG
    std::cout << "OrbWrapper::getORB() = " << this->orb
        << " done" << std::endl;
#endif
    return this->orb;
}

PortableServer::POA_var OrbWrapper::getRootPOA() {
#if DEBUG
    std::cout << "OrbWrapper::getRootPOA() ..." << std::endl;
#endif

#if DEBUG
    std::cout << "OrbWrapper::getRootPOA() = " << this->rootPOA
        << " done" << std::endl;
#endif
    return this->rootPOA;
}

void OrbWrapper::cleanup(CORBA::ORB_ptr orb, PortableServer::POA_ptr poa) {
#if DEBUG
    std::ostringstream orbSS;
    std::string orbString;
    if (CORBA::is_nil(orb)) {
        orbSS << "nil";
    } else {
        orbSS << orb;
    }
    orbString = orbSS.str();

    std::ostringstream poaSS;
    std::string poaString;
    if (CORBA::is_nil(poa)) {
        poaSS << "nil";
    } else {
        poaSS << poa;
    }
    poaString = poaSS.str();
    std::cout << "OrbWrapper::cleanup("
        << "orb=" << orbString
        << ", poa=" << poaString
        << ") ..." << std::endl;
#endif
    if (!CORBA::is_nil(poa)) {
        try {
            poa->destroy(1, 1);
        } catch (const CORBA::Exception& ex) {
            std::cerr << "CORBA::Exception caught while calling "
                << "poa->destroy()" << std::endl;
        } catch (...) {
            std::cerr << "Exception caught while calling "
                << "poa->destroy()" << std::endl;
        }
    }

    if (!CORBA::is_nil(orb)) {
        try {
            orb->shutdown(true);
        } catch (const CORBA::Exception& ex) {
            std::cerr << "CORBA::Exception caught while calling "
                << "orb->shutdown(true)" << std::endl;
        } catch (...) {
            std::cerr << "Exception caught while calling "
                << "orb->shutdown(true)" << std::endl;
        }

        try {
            orb->destroy();
        } catch (const CORBA::Exception& ex) {
            std::cerr << "CORBA::Exception caught while calling "
                << "orb->destroy()" << std::endl;
        } catch (...) {
            std::cerr << "Exception caught while calling "
                << "orb->destroy()" << std::endl;
        }
    }

#if DEBUG
    std::cout << "OrbWrapper::cleanup() done" << std::endl;
#endif
}
