//===============================================================
//
// $Source: /pv/CvsTree/pv/gen/src/prg/shimserver/OrbWrapper.hh,v $
//
// Copyright (c) 2009
//
// Bruker BioSpin MRI GmbH
// D-76275 Ettlingen, Germany
//
// All Rights Reserved
//===============================================================

#if !defined ( OrbWrapper_HH )
#define OrbWrapper_HH

//---------------------------------------------------------------
// includes
//---------------------------------------------------------------
#include <omniORB4/CORBA.h>

/**
 * A wrapper class for a <code>CORBA::ORB</code>.
 */
class OrbWrapper {
private:
    //-------------------------------------------------------------
    // constructors
    //-------------------------------------------------------------
    /**
     * Create a new OrbWrapper.
     * @param orb       The <code>CORBA::ORB</code> to be wrapped.
     * @param rootPOA   The (root) <code>PortableServer::POA</code>
     *                  to be wrapped.
     */
    OrbWrapper(CORBA::ORB_ptr orb, PortableServer::POA_ptr rootPOA);

public:
	//-------------------------------------------------------------
	// destructors
	//-------------------------------------------------------------
	/**
	 * Destroys this OrbWrapper.
	 */
  	virtual ~OrbWrapper();

    //-------------------------------------------------------------
    // static (factory) methods
    //-------------------------------------------------------------
    /**
     * A factory method creating a wrapped ORB instance.
     * @param nameserver        The corbaloc URL of the nameserver to be used.
     *                          If <code>NULL</code>, then the default
     *                          nameserver returned from
     *                          <code>CoreSystem::GetDefaultNameService()</code>
     *                          will be used.
     * @param endpoint          The optional endpoint specification, set to
     *                          <code>NULL</code> to use the default.
     * @param outConScanPeriod  The idle timeout in seconds for outgoing
     *                          (i.e. client initiated) connections.
     *                          If a connection has been idle for this amount
     *                          of time, the ORB closes it.
     *                          (omniORB default = 120).
     * @param inConScanPeriod   The idle timeout in seconds for incoming
     *                          connections.
     *                          If a connection has been idle for this amount
     *                          of time, the ORB closes it.
     *                          (omniORB default = 180).
     * @param scanGranularity   The granularity in seconds at which the ORB
     *                          performs scans for idle incoming and outgoing
     *                          connections. A value of zero turns off the
     *                          scanning altogether.
     *                          (omniORB default = 5).
     * @param clientCallTimeOutPeriod   The timeout in milliseconds for the
     *                          client side. If a call takes longer than the
     *                          specified number of milliseconds, the ORB
     *                          closes the connection to the server and raises
     *                          a TRANSIENT exception. A value of zero means
     *                          no timeout; calls can block for ever.
     *                          (omniORB default = 0).
     * @param clientConnectTimeOutPeriod    The timeout in milliseconds that is
     *                          used in the case that a new network connection
     *                          is established to the server. A value of zero
     *                          means that the normal call timeout is used.
     *                          (omniORB default = 0).
     * @return  An <code>OrbWrapper</code> instance upon success or
     *          <code>NULL</code> on error. The caller is responsible for
     *          deleting the returned object.
     */
    static OrbWrapper* createInstance(
        const char *nameserver,
        const char *endpoint,
        unsigned int outConScanPeriod,
        unsigned int inConScanPeriod,
        unsigned int scanGranularity,
        unsigned int clientCallTimeOutPeriod,
        unsigned int clientConnectTimeOutPeriod);

	//-------------------------------------------------------------
	// public member functions / methods
	//-------------------------------------------------------------

    /**
     * Get the <code>CORBA::ORB</code> instance wrapped by this class.
     * @return  The <code>CORBA::ORB</code> instance.
     */
    CORBA::ORB_var  getORB();

    /**
     * Get the (root) <code>PortableServer::POA</code> instance wrapped
     * by this class.
     * @return  The <code>PortableServer::POA</code> instance.
     */
    PortableServer::POA_var getRootPOA();

private:
    /**
     * Shutdown and/or destroy the given orb and poa (if not nil).
     */
    static void cleanup(CORBA::ORB_ptr orb, PortableServer::POA_ptr poa);

    /**
     * The wrapped <code>CORBA::ORB</code> instance.
     */
    CORBA::ORB_var orb;

    /**
     * The wrapped (root) <code>PortableServer::POA</code> instance.
     */
    PortableServer::POA_var rootPOA;
};
#endif
